dafny_runtime

Module _System

Re-exports

Macros

Structs

  • Configuration for formatting.
  • A single-threaded reference-counting pointer. ‘Rc’ stands for ‘Reference Counted’.

Enums

Traits

  • Used to do a cheap reference-to-reference conversion.
  • ? formatting.
  • A trait for giving a type a useful default value.
  • Trait for comparisons corresponding to equivalence relations.
  • A hashable type.
  • A trait for hashing an arbitrary stream of bytes.

Type Aliases

  • The type returned by formatter methods.

Derive Macros

  • Derive macro generating an impl of the trait Debug.
  • Derive macro generating an impl of the trait Default.
  • Derive macro generating an impl of the trait Eq.
  • Derive macro generating an impl of the trait Hash.